#include		"big5con.h"

extern struct fontInfo fi;
struct fontInfo fi;

int 
main(int argc, char *argv[])
{
    FILE        *fp;
    char	buf[256];
    u_char *	fdata;

    fp = fopen(argv[1], "r");
    fdata = (u_char *)FontLoadBdf(fp);
    argv[1][strlen(argv[1])-4] = 0;
    sprintf(buf, "%s.smf", argv[1]);
    fp = fopen(buf, "w+");
    fwrite(&fi, sizeof(struct fontInfo), 1, fp);
    fwrite(fdata, fi.size, 1, fp);
    fclose(fp);
    return 0;
}
