/*
 * KON - Kanji ON Linux Console - Copyright (C) 1992, 1993 Takashi MANABE
 * (manabe@tut.ac.jp)
 * 
 * KON is free software; you can redistribute it and/or modify it under the
 * terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * KON is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 675
 * Mass Ave, Cambridge, MA 02139, USA.
 */

#include	"big5con.h"

extern struct fontInfo fi;
extern          forceLoad;

u_char         *
FontLoadSmf(fp)
    FILE           *fp;
{
    u_char         *fdata = NULL;
    fread(&fi, sizeof(struct fontInfo), 1, fp);
    if ((fdata = (u_char *) malloc(fi.size)) == NULL)
	return NULL;
    fread(fdata, fi.size, 1, fp);
    return fdata;
}
