
/*
 * Copyright (C) 1995	Edward Der-Hua Liu, Hsin-Chu, Taiwan
 */

#include "big5con.h"

/* this function is used to avoid 4-byte bus-alignment */
u_long 
CONVT(char *s)
{
    u_long          kk;
    memcpy(&kk, s, 4);
    return kk;
}
