/*
	KON - Kanji ON Linux Console -
	Copyright (C) 1992, 1993 Takashi MANABE (manabe@tut.ac.jp)

	KON is free software; you can redistribute it and/or modify it
	under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	KON is distributed in the hope that it will be useful, but
	WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
	See the GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

/* mouse.h -- mouse driver interface */
#ifndef	MOUSE_H
#define	MOUSE_H

#include	<defs.h>

#define MOUSE_LFT	4
#define MOUSE_MID	2
#define MOUSE_RGT	1
#define	MOUSE_LIFETIME	55		 /* 5.5 seconds */

struct mouseInfo {
	bool	has_mouse;
	char	x, y,
		dx, dy,
		sx, sy,
		sw,
		stat;
};

extern struct mouseInfo	mInfo;
extern int mouseFd;

extern void	MouseInit(void);
extern int	MouseStart(void);
extern void	MouseGetPacket(u_char *, int);
extern void	MouseCleanup(void);

#endif
