/*
	KON - Kanji ON Linux Console - errors.h
	Copyright (C) 1993 by MAEDA Atusi (mad@math.keio.ac.jp)

	KON is free software; you can redistribute it and/or modify it
	under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	KON is distributed in the hope that it will be useful, but
	WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
	See the GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifndef	ERRORS_H
#define	ERRORS_H

extern void	Perror(const char *message);		/* perror(message) */
extern void	PerrorExit(const char *message);	/* perror(message) and die */
extern void	fatal(const char *format, ...);		/* print error message and die */
extern void	error(const char *format, ...);		/* print error message */
extern void	warn(const char *format, ...);		/* print warning message */
extern void	message(const char *format, ...);	/* print message */

#endif
