#ifndef BIG5CON_H
#define BIG5CON_H

#include        <stdio.h>
#include        <stdlib.h>
#include        <string.h>
#include        <unistd.h>
#include 	<ctype.h>
#include        <fcntl.h>
#include        <signal.h>
#include        <pwd.h>
#include        <grp.h>
#include        <utmp.h>
#include        <stdarg.h>
#include 	<dirent.h>

#include        <sys/types.h>
#include        <sys/ipc.h>
#include        <sys/shm.h>
#include        <sys/time.h>
#include        <sys/file.h>
#include        <sys/socket.h>
#include        <sys/mman.h>
#include        <sys/wait.h>
#include 	<sys/stat.h>
#include        <sys/ioctl.h>

#if defined(__FreeBSD__)
#include        <termios.h>
#include        <errno.h>
#include        <vm/vm_param.h>
#define termio termios
#include        <osreldate.h>
#if __FreeBSD_version >= 410000
#include        <sys/consio.h>
#include        <sys/kbio.h>
#else
#include        <machine/console.h>
#endif                          /* endif __FreeBSD_version >= 410000 */
#endif				/* endif __FreeBSD__ */

#if defined(linux)
#include        <linux/mm.h>
#include        <sys/kd.h>
#include        <termio.h>
#include        <sys/vt.h>
#include        <sys/kd.h>
#endif

#include 	"big5con.h"
#include 	"child.h"
#include 	"defs.h"
#include 	"errors.h"
#include 	"fnld.h"
#include 	"getcap.h"
#include 	"interface.h"
#include 	"ksymdef.h"
#include 	"mem.h"
#include 	"mouse.h"
#include 	"pho.h"
#include 	"setutmp.h"
#include 	"sock.h"
#include 	"state.h"
#include 	"term.h"
#include 	"vc.h"
#include 	"vga.h"
#include 	"vt.h"
#include 	"xi.h"

#define VERSION "0.92i"

#endif

